#!/usr/bin/env python3
# encoding: utf-8

# Unit tests for decopy
# Copyright (C) 2016 Maximiliano Curia <maxy@debian.org>
#
# License: ISC
#  Permission to use, copy, modify, and/or distribute this software for any
#  purpose with or without fee is hereby granted, provided that the above
#  copyright notice and this permission notice appear in all copies.
#  .
#  THE SOFTWARE IS PROVIDED "AS IS" AND ISC DISCLAIMS ALL WARRANTIES WITH
#  REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
#  AND FITNESS. IN NO EVENT SHALL ISC BE LIABLE FOR ANY SPECIAL, DIRECT,
#  INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
#  LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
#  OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
#  PERFORMANCE OF THIS SOFTWARE.

import json
import os
import unittest

try:
    import regex as re
except ImportError:
    import re

from decopy import matchers


class TestMatchers(unittest.TestCase):

    def testCleanComments(self):
        test = ('\n'
                'c Fortran\n'
                'C Idem\n'
                '  /* C // C++ */\n'
                ';; Lisp\n'
                '  <!-- HTML -->\n'
                'dnl m4\n'
                '  * continuation *\n'
                '\n')

        result = matchers.clean_comments(test)
        self.assertEqual(result, ' Fortran Idem C C++ Lisp HTML m4 '
                         'continuation ')

    def testParseCopyright(self):
        line = 'Copyright: Foo Bar <foo@example.com>'
        continuation = None
        copyrights, continuation = matchers.parse_copyright(line, continuation)
        self.assertEqual(len(copyrights), 1)
        self.assertEqual(copyrights[0], 'Foo Bar <foo@example.com>')
        line = '           2050- Baz <baz@example.com>'
        copyrights, continuation = matchers.parse_copyright(line, continuation)
        self.assertEqual(len(copyrights), 1)
        self.assertEqual(copyrights[0], '2050- Baz <baz@example.com>')
        line = 'Copyright (c) William Boo, (c) La Momia'
        copyrights, continuation = matchers.parse_copyright(line, continuation)
        self.assertEqual(len(copyrights), 2)
        self.assertEqual(copyrights[0], 'William Boo')
        self.assertEqual(copyrights[1], 'La Momia')

    def testParseHolders(self):
        text = (' /* Copyright Foo Bar\n'
                '  *  Baz */\n'
                ' // Copyright (c) Lorem, (c) Ipsum')
        holders = matchers.parse_holders(text)
        self.assertEqual([holder.person for holder in holders],
                         ['Foo Bar', 'Baz', 'Lorem', 'Ipsum'])

    def testParseHoldersMultiline(self):
        # Positive
        text = (' /* This software is Copyright:                 *\n'
                '  *  2018, Testy McTestface <testy@example.com> *\n'
                '  *  2017, Boaty McBoatface <boaty@example.com> */')
        holders = matchers.parse_holders(text)
        self.assertEqual([(str(h.years), h.name, h.email) for h in holders],
                         [('2018', 'Testy McTestface', 'testy@example.com'),
                          ('2017', 'Boaty McBoatface', 'boaty@example.com')])
        # Negative
        text = (' /* This software is free software:             *\n'
                '  *  2018, Testy McTestface <testy@example.com> *\n'
                '  *  2017, Boaty McBoatface <boaty@example.com> */')
        holders = matchers.parse_holders(text)
        self.assertEqual([(str(h.years), h.name, h.email) for h in holders],
                         [])

    def testParseHoldersMultilineYears(self):

        # Positive
        text = (' /* The software and docs is Copyright 1984, 1985, 1986 *\n'
                '  * 1987 Testy McTestface <testy@example.com>           *\n'
                '  * 1988 Boaty McBoatface <boaty@example.com>           */')
        holders = matchers.parse_holders(text)
        self.assertEqual(
            [(str(h.years), h.name, h.email) for h in holders],
            [('1984-1987', 'Testy McTestface', 'testy@example.com'),
             ('1988', 'Boaty McBoatface', 'boaty@example.com')])

        # Negative
        text = (' /* The software and docs is Copyright 1984, 1985, 1986 *\n'
                '  *                                                     *\n'
                '  * 1987 Testy McTestface <testy@example.com>           *\n'
                '  * 1988 Boaty McBoatface <boaty@example.com>           */')
        holders = matchers.parse_holders(text)
        self.assertEqual(
            [(str(h.years), h.name, h.email) for h in holders],
            [('1987', 'Testy McTestface', 'testy@example.com'),
             ('1988', 'Boaty McBoatface', 'boaty@example.com')])

    def _check_license_match(self, license_, matcher, expected, ignore=None):

        for key in ('licenseText', 'standardLicenseHeader',
                    'licenseExceptionText'):
            if key not in license_ or not license_[key]:
                continue
            if ignore and key in ignore:
                continue
            text = license_[key]
            result = matcher(text)
            self.assertEqual(result, expected,
                             msg='with key={}'.format(key))

    def _check_license_match_re(self, license_, matcher, expected):

        for key in ('licenseText', 'standardLicenseHeader',
                    'licenseExceptionText'):
            if key not in license_ or not license_[key]:
                continue
            text = license_[key]
            result = matcher(text)
            matches = re.match(expected, result)
            self.assertTrue(
                matches, msg='with key={}, result ({}) doesn\'t match with '
                'the expected value ({})\n{}'.format(
                    key, result, expected, text))

    @staticmethod
    def _find_licenses_matcher(text):
        licenses = matchers.find_licenses(
            matchers.clean_comments(text))
        return ' or '.join(sorted(licenses))

    def _open_license_and_check(self, filename, matcher, expected, ignore=None):

        with open(os.path.join('tests/testdata/licenses', filename)) as fp:
            license_ = json.load(fp)
        return self._check_license_match(license_, matcher, expected, ignore)

    def _open_license_and_check_re(self, filename, matcher, expected):

        with open(os.path.join('tests/testdata/licenses', filename)) as fp:
            license_ = json.load(fp)
        return self._check_license_match_re(license_, matcher, expected)

    def testParseBsd(self):

        def _bsd_matcher(text):
            return matchers.parse_bsd(
                matchers.clean_comments(text),
                None,
                'BSD')

        self._open_license_and_check('BSD-2-Clause.json',
                                     _bsd_matcher,
                                     'BSD-2-clause')
        self._open_license_and_check('BSD-2-Clause-FreeBSD.json',
                                     _bsd_matcher,
                                     'BSD-2-clause-FreeBSD')
        self._open_license_and_check('BSD-2-Clause-NetBSD.json',
                                     _bsd_matcher,
                                     'BSD-2-clause-NetBSD')
        self._open_license_and_check('BSD-3-Clause.json',
                                     _bsd_matcher,
                                     'BSD-3-clause')
        self._open_license_and_check('BSD-3-Clause-LBNL.json',
                                     _bsd_matcher,
                                     'BSD-3-clause')
        self._open_license_and_check('BSD-3-Clause-Clear.json',
                                     _bsd_matcher,
                                     'BSD-3-clause')
        self._open_license_and_check('BSD-3-Clause-No-Nuclear-License.json',
                                     _bsd_matcher,
                                     'BSD-3-clause-no-nuclear')
        self._open_license_and_check('BSD-3-Clause-No-Nuclear-License-2014.json',
                                     _bsd_matcher,
                                     'BSD-3-clause-no-nuclear')
        self._open_license_and_check('BSD-3-Clause-No-Nuclear-Warranty.json',
                                     _bsd_matcher,
                                     'BSD-3-clause-no-nuclear')
        self._open_license_and_check('BSD-4-Clause.json',
                                     _bsd_matcher,
                                     'BSD-4-clause')
        self._open_license_and_check('BSD-4-Clause-UC.json',
                                     _bsd_matcher,
                                     'BSD-4-clause')
        self._open_license_and_check('BSD-Protection.json',
                                     _bsd_matcher,
                                     'BSD-protection')
        self._open_license_and_check('BSD-Source-Code.json',
                                     _bsd_matcher,
                                     'BSD-source-code')
        self._open_license_and_check('BSD-3-Clause-Attribution.json',
                                     _bsd_matcher,
                                     'BSD-3-clause-attribution')
        self._open_license_and_check('0BSD.json',
                                     _bsd_matcher,
                                     '0BSD')

    def testParseCCBY(self):

        def _cc_by_matcher(text):
            return matchers.parse_cc_by(
                matchers.clean_comments(text),
                None,
                'CC-BY')

        self._open_license_and_check('CC-BY-1.0.json',
                                     _cc_by_matcher,
                                     'CC-BY-1')
        self._open_license_and_check('CC-BY-2.0.json',
                                     _cc_by_matcher,
                                     'CC-BY-2')
        self._open_license_and_check('CC-BY-2.5.json',
                                     _cc_by_matcher,
                                     'CC-BY-2.5')
        self._open_license_and_check('CC-BY-3.0.json',
                                     _cc_by_matcher,
                                     'CC-BY-3')
        self._open_license_and_check('CC-BY-4.0.json',
                                     _cc_by_matcher,
                                     'CC-BY-4')
        self._open_license_and_check('CC-BY-NC-1.0.json',
                                     _cc_by_matcher,
                                     'CC-BY-NC-1')
        self._open_license_and_check('CC-BY-NC-2.0.json',
                                     _cc_by_matcher,
                                     'CC-BY-NC-2')
        self._open_license_and_check('CC-BY-NC-2.5.json',
                                     _cc_by_matcher,
                                     'CC-BY-NC-2.5')
        self._open_license_and_check('CC-BY-NC-3.0.json',
                                     _cc_by_matcher,
                                     'CC-BY-NC-3')
        self._open_license_and_check('CC-BY-NC-4.0.json',
                                     _cc_by_matcher,
                                     'CC-BY-NC-4')
        self._open_license_and_check('CC-BY-NC-ND-1.0.json',
                                     _cc_by_matcher,
                                     'CC-BY-NC-ND-1')
        self._open_license_and_check('CC-BY-NC-ND-2.0.json',
                                     _cc_by_matcher,
                                     'CC-BY-NC-ND-2')
        self._open_license_and_check('CC-BY-NC-ND-2.5.json',
                                     _cc_by_matcher,
                                     'CC-BY-NC-ND-2.5')
        self._open_license_and_check('CC-BY-NC-ND-3.0.json',
                                     _cc_by_matcher,
                                     'CC-BY-NC-ND-3')
        self._open_license_and_check('CC-BY-NC-ND-4.0.json',
                                     _cc_by_matcher,
                                     'CC-BY-NC-ND-4')
        self._open_license_and_check('CC-BY-NC-SA-1.0.json',
                                     _cc_by_matcher,
                                     'CC-BY-NC-SA-1')
        self._open_license_and_check('CC-BY-NC-SA-2.0.json',
                                     _cc_by_matcher,
                                     'CC-BY-NC-SA-2')
        self._open_license_and_check('CC-BY-NC-SA-2.5.json',
                                     _cc_by_matcher,
                                     'CC-BY-NC-SA-2.5')
        self._open_license_and_check('CC-BY-NC-SA-3.0.json',
                                     _cc_by_matcher,
                                     'CC-BY-NC-SA-3')
        self._open_license_and_check('CC-BY-NC-SA-4.0.json',
                                     _cc_by_matcher,
                                     'CC-BY-NC-SA-4')
        self._open_license_and_check('CC-BY-ND-1.0.json',
                                     _cc_by_matcher,
                                     'CC-BY-ND-1')
        self._open_license_and_check('CC-BY-ND-2.0.json',
                                     _cc_by_matcher,
                                     'CC-BY-ND-2')
        self._open_license_and_check('CC-BY-ND-2.5.json',
                                     _cc_by_matcher,
                                     'CC-BY-ND-2.5')
        self._open_license_and_check('CC-BY-ND-3.0.json',
                                     _cc_by_matcher,
                                     'CC-BY-ND-3')
        self._open_license_and_check('CC-BY-ND-4.0.json',
                                     _cc_by_matcher,
                                     'CC-BY-ND-4')
        self._open_license_and_check('CC-BY-SA-1.0.json',
                                     _cc_by_matcher,
                                     'CC-BY-SA-1')
        self._open_license_and_check('CC-BY-SA-2.0.json',
                                     _cc_by_matcher,
                                     'CC-BY-SA-2')
        self._open_license_and_check('CC-BY-SA-2.5.json',
                                     _cc_by_matcher,
                                     'CC-BY-SA-2.5')
        self._open_license_and_check('CC-BY-SA-3.0.json',
                                     _cc_by_matcher,
                                     'CC-BY-SA-3')
        self._open_license_and_check('CC-BY-SA-4.0.json',
                                     _cc_by_matcher,
                                     'CC-BY-SA-4')

        self._open_license_and_check('CC0-1.0.json',
                                     self._find_licenses_matcher,
                                     'CC0')

    def testParseGPL(self):

        def _gpl_matcher(text):
            return matchers.parse_gpl(
                matchers.clean_comments(text),
                None,
                'GPL')

        self._open_license_and_check('AGPL-1.0.json',
                                     self._find_licenses_matcher,
                                     'AGPL-1')
        self._open_license_and_check_re('AGPL-3.0.json',
                                        self._find_licenses_matcher,
                                        r'AGPL-3\+?$')
        self._open_license_and_check_re('GPL-1.0.json',
                                        self._find_licenses_matcher,
                                        r'GPL-1\+?$')
        self._open_license_and_check('GPL-1.0+.json',
                                     self._find_licenses_matcher,
                                     'GPL-1+')
        self._open_license_and_check_re('GPL-2.0.json',
                                        self._find_licenses_matcher,
                                        r'GPL-2\+?$')
        self._open_license_and_check('GPL-2.0+.json',
                                     self._find_licenses_matcher,
                                     'GPL-2+')
        self._open_license_and_check_re('GPL-3.0.json',
                                        self._find_licenses_matcher,
                                        r'GPL(?:-3\+)?$')
        self._open_license_and_check('GPL-3.0+.json',
                                     self._find_licenses_matcher,
                                     'GPL-3+')

        self._open_license_and_check('GPL-2.0-with-autoconf-exception.json',
                                     _gpl_matcher,
                                     'GPL with AutoConf exception')
        self._open_license_and_check('GPL-2.0-with-bison-exception.json',
                                     _gpl_matcher,
                                     'GPL with Bison exception')
        self._open_license_and_check('GPL-2.0-with-classpath-exception.json',
                                     _gpl_matcher,
                                     'GPL with ClassPath exception')
        self._open_license_and_check('GPL-2.0-with-GCC-exception.json',
                                     _gpl_matcher,
                                     'GPL with GCC exception')

        self._open_license_and_check('GPL-3.0-with-autoconf-exception.json',
                                     _gpl_matcher,
                                     'GPL-3 with AutoConf exception')
        self._open_license_and_check('GPL-3.0-with-GCC-exception.json',
                                     _gpl_matcher,
                                     'GPL-3 with GCC exception')

        self._open_license_and_check('389-exception.json',
                                     _gpl_matcher,
                                     'GPL-2 with 389 exception')

        self._open_license_and_check('Autoconf-exception-2.0.json',
                                     _gpl_matcher,
                                     'GPL with AutoConf exception')
        self._open_license_and_check('Autoconf-exception-3.0.json',
                                     _gpl_matcher,
                                     'GPL-3 with AutoConf exception')
        self._open_license_and_check('Bison-exception-2.2.json',
                                     _gpl_matcher,
                                     'GPL with Bison exception')
        self._open_license_and_check('Classpath-exception-2.0.json',
                                     _gpl_matcher,
                                     'GPL with ClassPath exception')
        self._open_license_and_check('CLISP-exception-2.0.json',
                                     _gpl_matcher,
                                     'GPL-2 with CLISP exception')
        self._open_license_and_check('DigiRule-FOSS-exception.json',
                                     _gpl_matcher,
                                     'GPL with DigiRule-FOSS exception')
        self._open_license_and_check('eCos-exception-2.0.json',
                                     _gpl_matcher,
                                     'GPL with eCos-2 exception')
        self._open_license_and_check('Fawkes-Runtime-exception.json',
                                     _gpl_matcher,
                                     'GPL with ClassPath_MIF exception')
        self._open_license_and_check('FLTK-exception.json',
                                     _gpl_matcher,
                                     'GPL with FLTK exception')
        self._open_license_and_check('Font-exception-2.0.json',
                                     _gpl_matcher,
                                     'GPL with Font exception')
        self._open_license_and_check('freertos-exception-2.0.json',
                                     _gpl_matcher,
                                     'GPL with FreeRTOS exception')
        self._open_license_and_check('GCC-exception-2.0.json',
                                     _gpl_matcher,
                                     'GPL with GCC exception')
        self._open_license_and_check('GCC-exception-3.1.json',
                                     _gpl_matcher,
                                     'GPL-3 with GCC exception')
        self._open_license_and_check('gnu-javamail-exception.json',
                                     _gpl_matcher,
                                     'GPL with GNU-JavaMail exception')
        self._open_license_and_check('i2p-gpl-java-exception.json',
                                     _gpl_matcher,
                                     'GPL with i2p-Java exception')
        self._open_license_and_check('Libtool-exception.json',
                                     _gpl_matcher,
                                     'GPL with LibTool exception')
        self._open_license_and_check('LZMA-exception.json',
                                     _gpl_matcher,
                                     'GPL with LZMA exception')
        self._open_license_and_check('mif-exception.json',
                                     _gpl_matcher,
                                     'GPL with MIF exception')
        self._open_license_and_check('Nokia-Qt-exception-1.1.json',
                                     _gpl_matcher,
                                     'GPL-2.1 with Qt1.1 exception')
        self._open_license_and_check('OCCT-exception-1.0.json',
                                     _gpl_matcher,
                                     'GPL-2.1 with OCCT-1 exception')
        self._open_license_and_check('openvpn-openssl-exception.json',
                                     _gpl_matcher,
                                     'GPL with openvpn-openssl exception')
        # handled in the find licenses part
        # FIXME: ?
        self._open_license_and_check('Qwt-exception-1.0.json',
                                     self._find_licenses_matcher,
                                     'LGPL-2.1 with Qwt-1 exception')
        self._open_license_and_check('u-boot-exception-2.0.json',
                                     _gpl_matcher,
                                     'GPL with u-boot exception')
        self._open_license_and_check('WxWindows-exception-3.1.json',
                                     _gpl_matcher,
                                     'GPL with WxWindows-3.1+ exception')

    def testParseLGPL(self):

        self._open_license_and_check_re('LGPL-2.0.json',
                                        self._find_licenses_matcher,
                                        r'LGPL-2\+?')
        self._open_license_and_check('LGPL-2.0+.json',
                                     self._find_licenses_matcher,
                                     'LGPL-2+')
        self._open_license_and_check_re('LGPL-2.1.json',
                                        self._find_licenses_matcher,
                                        r'LGPL-2.1\+?')
        self._open_license_and_check('LGPL-2.1+.json',
                                     self._find_licenses_matcher,
                                     'LGPL-2.1+')
        self._open_license_and_check_re('LGPL-3.0.json',
                                        self._find_licenses_matcher,
                                        r'LGPL-3\+?')
        self._open_license_and_check('LGPL-3.0+.json',
                                     self._find_licenses_matcher,
                                     'LGPL-3+')
        self._open_license_and_check('LGPLLR.json',
                                     self._find_licenses_matcher,
                                     'LGPLLR')

    def testParseGFDL(self):

        self._open_license_and_check('GFDL-1.1.json',
                                     self._find_licenses_matcher,
                                     'GFDL-1.1')
        self._open_license_and_check('GFDL-1.2.json',
                                     self._find_licenses_matcher,
                                     'GFDL-NIV-1.2')
        self._open_license_and_check('GFDL-1.3.json',
                                     self._find_licenses_matcher,
                                     'GFDL-NIV-1.3')

    def testParseLPPL(self):

        self._open_license_and_check('LPPL-1.0.json',
                                     self._find_licenses_matcher,
                                     'LPPL-1+')
        self._open_license_and_check('LPPL-1.1.json',
                                     self._find_licenses_matcher,
                                     'LPPL-1.1+')
        self._open_license_and_check('LPPL-1.2.json',
                                     self._find_licenses_matcher,
                                     'LPPL-1.2+')
        self._open_license_and_check('LPPL-1.3a.json',
                                     self._find_licenses_matcher,
                                     'LPPL-1.3+')
        self._open_license_and_check('LPPL-1.3c.json',
                                     self._find_licenses_matcher,
                                     'LPPL-1.3+')

    def testParseMIT(self):

        self._open_license_and_check('MIT.json',
                                     self._find_licenses_matcher,
                                     'Expat')
        self._open_license_and_check('MIT-advertising.json',
                                     self._find_licenses_matcher,
                                     'MIT-advertising')
        self._open_license_and_check('MIT-CMU.json',
                                     self._find_licenses_matcher,
                                     'MIT-CMU')
        self._open_license_and_check('MIT-enna.json',
                                     self._find_licenses_matcher,
                                     'MIT-enna')
        self._open_license_and_check('MIT-feh.json',
                                     self._find_licenses_matcher,
                                     'MIT-feh')
        self._open_license_and_check('MITNFA.json',
                                     self._find_licenses_matcher,
                                     'MITNFA')

    def testParseZPL(self):

        self._open_license_and_check('ZPL-1.1.json',
                                     self._find_licenses_matcher,
                                     'ZPL-1.1')
        self._open_license_and_check('ZPL-2.0.json',
                                     self._find_licenses_matcher,
                                     'ZPL-2')
        self._open_license_and_check('ZPL-2.1.json',
                                     self._find_licenses_matcher,
                                     'ZPL-2.1')


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python3
# encoding: utf-8

# Unit tests for decopy
# Copyright (C) 2018 Maximiliano Curia <maxy@debian.org>
#
# License: ISC
#  Permission to use, copy, modify, and/or distribute this software for any
#  purpose with or without fee is hereby granted, provided that the above
#  copyright notice and this permission notice appear in all copies.
#  .
#  THE SOFTWARE IS PROVIDED "AS IS" AND ISC DISCLAIMS ALL WARRANTIES WITH
#  REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
#  AND FITNESS. IN NO EVENT SHALL ISC BE LIABLE FOR ANY SPECIAL, DIRECT,
#  INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
#  LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
#  OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
#  PERFORMANCE OF THIS SOFTWARE.

import json
import os
import unittest

from decopy import matchers


class TestSpdx(unittest.TestCase):

    def _check_license_match(self, license_, matcher, expected, ignore=None):

        for key in ('licenseText', 'standardLicenseHeader',
                    'licenseExceptionText'):
            if key not in license_ or not license_[key]:
                continue
            if ignore and key in ignore:
                continue
            text = license_[key]
            result = matcher(text)
            self.assertEqual(result, expected,
                             msg='with key={}'.format(key))

    def _open_license_and_check(self, filename, matcher, expected, ignore=None):

        with open(os.path.join('tests/testdata/licenses', filename)) as fp:
            license_ = json.load(fp)
        return self._check_license_match(license_, matcher, expected, ignore)

    @staticmethod
    def _find_licenses_matcher(text):
        licenses = matchers.find_licenses(
            matchers.clean_comments(text))
        return ' or '.join(sorted(licenses))

    def testSpdxA(self):

        self._open_license_and_check('AAL.json',
                                     self._find_licenses_matcher,
                                     'AAL')
        self._open_license_and_check('Abstyles.json',
                                     self._find_licenses_matcher,
                                     'Abstyles')
        self._open_license_and_check('Adobe-2006.json',
                                     self._find_licenses_matcher,
                                     'Adobe-2006')
        self._open_license_and_check('Adobe-Glyph.json',
                                     self._find_licenses_matcher,
                                     'Adobe-Glyph')
        self._open_license_and_check('ADSL.json',
                                     self._find_licenses_matcher,
                                     'ADSL')
        self._open_license_and_check('AFL-1.1.json',
                                     self._find_licenses_matcher,
                                     'Academic-1.1')
        self._open_license_and_check('AFL-1.2.json',
                                     self._find_licenses_matcher,
                                     'Academic-1.2')
        self._open_license_and_check('AFL-2.0.json',
                                     self._find_licenses_matcher,
                                     'Academic-2')
        self._open_license_and_check('AFL-2.1.json',
                                     self._find_licenses_matcher,
                                     'Academic-2.1')
        self._open_license_and_check('AFL-3.0.json',
                                     self._find_licenses_matcher,
                                     'Academic-3')
        self._open_license_and_check('Afmparse.json',
                                     self._find_licenses_matcher,
                                     'Afmparse')
        self._open_license_and_check('Aladdin.json',
                                     self._find_licenses_matcher,
                                     'Aladdin')
        self._open_license_and_check('AMDPLPA.json',
                                     self._find_licenses_matcher,
                                     'AMDPLPA')
        self._open_license_and_check('AML.json',
                                     self._find_licenses_matcher,
                                     'AML')
        self._open_license_and_check('AMPAS.json',
                                     self._find_licenses_matcher,
                                     'BSD-2-clause-AMPAS')
        self._open_license_and_check('ANTLR-PD.json',
                                     self._find_licenses_matcher,
                                     'ANTLR-PD')
        self._open_license_and_check('Apache-1.0.json',
                                     self._find_licenses_matcher,
                                     'Apache-1')
        self._open_license_and_check('Apache-1.1.json',
                                     self._find_licenses_matcher,
                                     'Apache-1.1')
        self._open_license_and_check('Apache-2.0.json',
                                     self._find_licenses_matcher,
                                     'Apache-2')
        self._open_license_and_check('APAFML.json',
                                     self._find_licenses_matcher,
                                     'APAFML')
        self._open_license_and_check('APL-1.0.json',
                                     self._find_licenses_matcher,
                                     'APL-1',
                                     ignore={'standardLicenseHeader'})
        self._open_license_and_check('APSL-1.0.json',
                                     self._find_licenses_matcher,
                                     'APSL-1')
        self._open_license_and_check('APSL-1.1.json',
                                     self._find_licenses_matcher,
                                     'APSL-1.1')
        self._open_license_and_check('APSL-1.2.json',
                                     self._find_licenses_matcher,
                                     'APSL-1.2')
        self._open_license_and_check('APSL-2.0.json',
                                     self._find_licenses_matcher,
                                     'APSL-2')
        self._open_license_and_check('Artistic-1.0-cl8.json',
                                     self._find_licenses_matcher,
                                     'Artistic-1-cl8')
        self._open_license_and_check('Artistic-1.0-Perl.json',
                                     self._find_licenses_matcher,
                                     'Artistic-1-Perl')
        self._open_license_and_check('Artistic-1.0.json',
                                     self._find_licenses_matcher,
                                     'Artistic-1')
        self._open_license_and_check('Artistic-2.0.json',
                                     self._find_licenses_matcher,
                                     'Artistic-2')

    def testSpdxB(self):
        self._open_license_and_check('Bahyph.json',
                                     self._find_licenses_matcher,
                                     'Bahyph')
        self._open_license_and_check('Barr.json',
                                     self._find_licenses_matcher,
                                     'Barr')
        self._open_license_and_check('BitTorrent-1.0.json',
                                     self._find_licenses_matcher,
                                     'BitTorrent-1')
        # The header has the wrong version in it
        self._open_license_and_check('BitTorrent-1.1.json',
                                     self._find_licenses_matcher,
                                     'BitTorrent-1')
        self._open_license_and_check('Borceux.json',
                                     self._find_licenses_matcher,
                                     'Borceux')
        self._open_license_and_check('BSL-1.0.json',
                                     self._find_licenses_matcher,
                                     'BSL-1')
        self._open_license_and_check('bzip2-1.0.5.json',
                                     self._find_licenses_matcher,
                                     'BSD-like-bzip2')
        self._open_license_and_check('bzip2-1.0.6.json',
                                     self._find_licenses_matcher,
                                     'BSD-like-bzip2')

    def testSpdxC(self):
        self._open_license_and_check('Caldera.json',
                                     self._find_licenses_matcher,
                                     'Caldera')
        self._open_license_and_check('CATOSL-1.1.json',
                                     self._find_licenses_matcher,
                                     'CATOSL-1.1')
        self._open_license_and_check('CDDL-1.0.json',
                                     self._find_licenses_matcher,
                                     'CDDL-1')
        self._open_license_and_check('CDDL-1.1.json',
                                     self._find_licenses_matcher,
                                     'CDDL-1.1')
        self._open_license_and_check('CECILL-1.0.json',
                                     self._find_licenses_matcher,
                                     'CeCILL-1')
        self._open_license_and_check('CECILL-1.1.json',
                                     self._find_licenses_matcher,
                                     'CeCILL-1.1')
        self._open_license_and_check('CECILL-2.0.json',
                                     self._find_licenses_matcher,
                                     'CeCILL-2')
        self._open_license_and_check('CECILL-2.1.json',
                                     self._find_licenses_matcher,
                                     'CeCILL-2.1')
        self._open_license_and_check('CECILL-B.json',
                                     self._find_licenses_matcher,
                                     'CeCILL-B')
        self._open_license_and_check('CECILL-C.json',
                                     self._find_licenses_matcher,
                                     'CeCILL-C')

        self._open_license_and_check('ClArtistic.json',
                                     self._find_licenses_matcher,
                                     'ClArtistic')

        self._open_license_and_check('CNRI-Jython.json',
                                     self._find_licenses_matcher,
                                     'CNRI-Jython')
        self._open_license_and_check('CNRI-Python-GPL-Compatible.json',
                                     self._find_licenses_matcher,
                                     'CNRI-Python-GPL-Compatible')
        self._open_license_and_check('CNRI-Python.json',
                                     self._find_licenses_matcher,
                                     'CNRI-Python')

        self._open_license_and_check('Condor-1.1.json',
                                     self._find_licenses_matcher,
                                     'Condor-1.1')

        self._open_license_and_check('CPAL-1.0.json',
                                     self._find_licenses_matcher,
                                     'CPAL-1')
        self._open_license_and_check('CPL-1.0.json',
                                     self._find_licenses_matcher,
                                     'CPL-1')
        self._open_license_and_check('CPOL-1.02.json',
                                     self._find_licenses_matcher,
                                     'CPOL-1.02')
        self._open_license_and_check('Crossword.json',
                                     self._find_licenses_matcher,
                                     'Crossword')
        self._open_license_and_check('CrystalStacker.json',
                                     self._find_licenses_matcher,
                                     'CrystalStacker')
        self._open_license_and_check('CUA-OPL-1.0.json',
                                     self._find_licenses_matcher,
                                     'CUA-OPL-1')
        self._open_license_and_check('Cube.json',
                                     self._find_licenses_matcher,
                                     'Cube')
        self._open_license_and_check('curl.json',
                                     self._find_licenses_matcher,
                                     'curl')

    def testSpdxD(self):
        self._open_license_and_check('D-FSL-1.0.json',
                                     self._find_licenses_matcher,
                                     'D-FSL')
        self._open_license_and_check('diffmark.json',
                                     self._find_licenses_matcher,
                                     'diffmark')
        self._open_license_and_check('DOC.json',
                                     self._find_licenses_matcher,
                                     'DOC')
        self._open_license_and_check('Dotseqn.json',
                                     self._find_licenses_matcher,
                                     'Dotseqn')
        self._open_license_and_check('DSDP.json',
                                     self._find_licenses_matcher,
                                     'DSDP')
        self._open_license_and_check('dvipdfm.json',
                                     self._find_licenses_matcher,
                                     'dvipdfm')

    def testSpdxE(self):
        self._open_license_and_check('ECL-1.0.json',
                                     self._find_licenses_matcher,
                                     'ECL-1')
        self._open_license_and_check('ECL-2.0.json',
                                     self._find_licenses_matcher,
                                     'ECL-2')
        self._open_license_and_check('eCos-2.0.json',
                                     self._find_licenses_matcher,
                                     'GPL-2+ with eCos-2 exception')
        self._open_license_and_check('EFL-1.0.json',
                                     self._find_licenses_matcher,
                                     'EFL-1')
        self._open_license_and_check('EFL-2.0.json',
                                     self._find_licenses_matcher,
                                     'EFL-2')
        self._open_license_and_check('eGenix.json',
                                     self._find_licenses_matcher,
                                     'eGenix')
        self._open_license_and_check('Entessa.json',
                                     self._find_licenses_matcher,
                                     'BSD-like-Entessa')
        self._open_license_and_check('EPL-1.0.json',
                                     self._find_licenses_matcher,
                                     'EPL-1')
        self._open_license_and_check('ErlPL-1.1.json',
                                     self._find_licenses_matcher,
                                     'ErlPL-1.1')
        self._open_license_and_check('EUDatagrid.json',
                                     self._find_licenses_matcher,
                                     'EUDatagrid')
        self._open_license_and_check('EUPL-1.0.json',
                                     self._find_licenses_matcher,
                                     'EUPL-1')
        self._open_license_and_check('EUPL-1.1.json',
                                     self._find_licenses_matcher,
                                     'EUPL-1.1')
        self._open_license_and_check('Eurosym.json',
                                     self._find_licenses_matcher,
                                     'Eurosym')

    def testSpdxF(self):
        self._open_license_and_check('Fair.json',
                                     self._find_licenses_matcher,
                                     'Fair')
        self._open_license_and_check('Frameworx-1.0.json',
                                     self._find_licenses_matcher,
                                     'Frameworx-1')
        self._open_license_and_check('FreeImage.json',
                                     self._find_licenses_matcher,
                                     'FreeImage-1')
        self._open_license_and_check('FSFAP.json',
                                     self._find_licenses_matcher,
                                     'FSFAP')
        self._open_license_and_check('FSFUL.json',
                                     self._find_licenses_matcher,
                                     'FSFUL')
        self._open_license_and_check('FSFULLR.json',
                                     self._find_licenses_matcher,
                                     'FSFULLR')
        self._open_license_and_check('FTL.json',
                                     self._find_licenses_matcher,
                                     'FTL')

    def testSpdxG(self):
        self._open_license_and_check('GFDL-1.1.json',
                                     self._find_licenses_matcher,
                                     'GFDL-1.1')
        self._open_license_and_check('GFDL-1.2.json',
                                     self._find_licenses_matcher,
                                     'GFDL-NIV-1.2')
        self._open_license_and_check('GFDL-1.3.json',
                                     self._find_licenses_matcher,
                                     'GFDL-NIV-1.3')
        self._open_license_and_check('Giftware.json',
                                     self._find_licenses_matcher,
                                     'Giftware')
        self._open_license_and_check('GL2PS.json',
                                     self._find_licenses_matcher,
                                     'GL2PS')
        self._open_license_and_check('Glide.json',
                                     self._find_licenses_matcher,
                                     'Glide')
        self._open_license_and_check('Glulxe.json',
                                     self._find_licenses_matcher,
                                     'Glulxe')
        self._open_license_and_check('gnuplot.json',
                                     self._find_licenses_matcher,
                                     'gnuplot')
        self._open_license_and_check('gSOAP-1.3b.json',
                                     self._find_licenses_matcher,
                                     'gSOAP-1.3')

    def testSpdxH(self):
        self._open_license_and_check('HaskellReport.json',
                                     self._find_licenses_matcher,
                                     'HaskellReport')
        self._open_license_and_check('HPND.json',
                                     self._find_licenses_matcher,
                                     'HPND')

    def testSpdxI(self):
        self._open_license_and_check('IBM-pibs.json',
                                     self._find_licenses_matcher,
                                     'IBM-pibs')
        self._open_license_and_check('ICU.json',
                                     self._find_licenses_matcher,
                                     'MIT-like-icu')
        self._open_license_and_check('IJG.json',
                                     self._find_licenses_matcher,
                                     'IJG')
        self._open_license_and_check('ImageMagick.json',
                                     self._find_licenses_matcher,
                                     'ImageMagick')
        self._open_license_and_check('iMatix.json',
                                     self._find_licenses_matcher,
                                     'iMatix')
        self._open_license_and_check('Imlib2.json',
                                     self._find_licenses_matcher,
                                     'MIT-Imlib2')
        self._open_license_and_check('Info-ZIP.json',
                                     self._find_licenses_matcher,
                                     'Info-ZIP')
        self._open_license_and_check('Intel-ACPI.json',
                                     self._find_licenses_matcher,
                                     'Intel-ACPI')
        self._open_license_and_check('Intel.json',
                                     self._find_licenses_matcher,
                                     'Intel')
        self._open_license_and_check('Interbase-1.0.json',
                                     self._find_licenses_matcher,
                                     'Interbase-1')
        self._open_license_and_check('IPA.json',
                                     self._find_licenses_matcher,
                                     'IPA-1')
        self._open_license_and_check('IPL-1.0.json',
                                     self._find_licenses_matcher,
                                     'IPL-1')
        self._open_license_and_check('ISC.json',
                                     self._find_licenses_matcher,
                                     'ISC')

    def testSpdxJ(self):
        self._open_license_and_check('JasPer-2.0.json',
                                     self._find_licenses_matcher,
                                     'JasPer-2')
        self._open_license_and_check('JSON.json',
                                     self._find_licenses_matcher,
                                     'JSON')

    def testSpdxL(self):
        self._open_license_and_check('LAL-1.2.json',
                                     self._find_licenses_matcher,
                                     'LAL-1.2')
        self._open_license_and_check('LAL-1.3.json',
                                     self._find_licenses_matcher,
                                     'LAL-1.3')
        self._open_license_and_check('Latex2e.json',
                                     self._find_licenses_matcher,
                                     'Latex2e')
        self._open_license_and_check('Leptonica.json',
                                     self._find_licenses_matcher,
                                     'Leptonica')
        self._open_license_and_check('Libpng.json',
                                     self._find_licenses_matcher,
                                     'Libpng')
        self._open_license_and_check('libtiff.json',
                                     self._find_licenses_matcher,
                                     'libtiff')
        self._open_license_and_check('LiLiQ-P-1.1.json',
                                     self._find_licenses_matcher,
                                     'LiLiQ-P-1.1')
        self._open_license_and_check('LiLiQ-R-1.1.json',
                                     self._find_licenses_matcher,
                                     'LiLiQ-R-1.1')
        self._open_license_and_check('LiLiQ-Rplus-1.1.json',
                                     self._find_licenses_matcher,
                                     'LiLiQ-R+-1.1')
        self._open_license_and_check('LPL-1.0.json',
                                     self._find_licenses_matcher,
                                     'LPL-1')
        self._open_license_and_check('LPL-1.02.json',
                                     self._find_licenses_matcher,
                                     'LPL-1.02')

    def testSpdxM(self):
        self._open_license_and_check('MakeIndex.json',
                                     self._find_licenses_matcher,
                                     'MakeIndex')
        self._open_license_and_check('MirOS.json',
                                     self._find_licenses_matcher,
                                     'MirOS')
        self._open_license_and_check('Motosoto.json',
                                     self._find_licenses_matcher,
                                     'Motosoto',
                                     ignore={'standardLicenseHeader'})
        self._open_license_and_check('mpich2.json',
                                     self._find_licenses_matcher,
                                     'mpich')
        self._open_license_and_check('MPL-1.0.json',
                                     self._find_licenses_matcher,
                                     'MPL-1')
        self._open_license_and_check('MPL-1.1.json',
                                     self._find_licenses_matcher,
                                     'MPL-1.1')
        self._open_license_and_check('MPL-2.0-no-copyleft-exception.json',
                                     self._find_licenses_matcher,
                                     'MPL-2 with no-copyleft exception',
                                     ignore={'licenseText'})
        self._open_license_and_check('MPL-2.0.json',
                                     self._find_licenses_matcher,
                                     'MPL-2')
        self._open_license_and_check('MS-PL.json',
                                     self._find_licenses_matcher,
                                     'MS-PL')
        self._open_license_and_check('MS-RL.json',
                                     self._find_licenses_matcher,
                                     'MS-RL')
        self._open_license_and_check('MTLL.json',
                                     self._find_licenses_matcher,
                                     'MTLL')
        self._open_license_and_check('Multics.json',
                                     self._find_licenses_matcher,
                                     'Multics')
        self._open_license_and_check('Mup.json',
                                     self._find_licenses_matcher,
                                     'BSD-like-Mup')

    def testSpdxN(self):
        self._open_license_and_check('NASA-1.3.json',
                                     self._find_licenses_matcher,
                                     'NASA-1.3')
        self._open_license_and_check('Naumen.json',
                                     self._find_licenses_matcher,
                                     'Naumen')
        self._open_license_and_check('NBPL-1.0.json',
                                     self._find_licenses_matcher,
                                     'NBPL-1')
        self._open_license_and_check('NCSA.json',
                                     self._find_licenses_matcher,
                                     'NCSA')
        self._open_license_and_check('NetCDF.json',
                                     self._find_licenses_matcher,
                                     'NetCDF')
        self._open_license_and_check('Newsletr.json',
                                     self._find_licenses_matcher,
                                     'Newsletr')
        self._open_license_and_check('NGPL.json',
                                     self._find_licenses_matcher,
                                     'NGPL')
        self._open_license_and_check('NLOD-1.0.json',
                                     self._find_licenses_matcher,
                                     'NLOD-1')
        self._open_license_and_check('NLPL.json',
                                     self._find_licenses_matcher,
                                     'NLPL')
        self._open_license_and_check('Nokia.json',
                                     self._find_licenses_matcher,
                                     'NOKOS-1')
        self._open_license_and_check('NOSL.json',
                                     self._find_licenses_matcher,
                                     'NOSL-1')
        self._open_license_and_check('Noweb.json',
                                     self._find_licenses_matcher,
                                     'Noweb')
        self._open_license_and_check('NPL-1.0.json',
                                     self._find_licenses_matcher,
                                     'NPL-1')
        # The header included in the file is NPL-1
        self._open_license_and_check('NPL-1.1.json',
                                     self._find_licenses_matcher,
                                     'MPL-1.1_NPL-1.1',
                                     ignore={'standardLicenseHeader'})
        self._open_license_and_check('NPOSL-3.0.json',
                                     self._find_licenses_matcher,
                                     'NPOSL-3')
        self._open_license_and_check('NRL.json',
                                     self._find_licenses_matcher,
                                     'NRL')
        self._open_license_and_check('NTP.json',
                                     self._find_licenses_matcher,
                                     'NTP')
        self._open_license_and_check('Nunit.json',
                                     self._find_licenses_matcher,
                                     'Zlib-acknowledgement')

    def testSpdxO(self):
        self._open_license_and_check('OCCT-PL.json',
                                     self._find_licenses_matcher,
                                     'OCCT-PL')
        self._open_license_and_check('OCLC-2.0.json',
                                     self._find_licenses_matcher,
                                     'OCLC-2')
        self._open_license_and_check('ODbL-1.0.json',
                                     self._find_licenses_matcher,
                                     'ODbL-1')
        self._open_license_and_check('OFL-1.0.json',
                                     self._find_licenses_matcher,
                                     'OFL-1')
        self._open_license_and_check('OFL-1.1.json',
                                     self._find_licenses_matcher,
                                     'OFL-1.1')
        self._open_license_and_check('OGTSL.json',
                                     self._find_licenses_matcher,
                                     'OGTSL')
        self._open_license_and_check('OLDAP-1.1.json',
                                     self._find_licenses_matcher,
                                     'OLDAP-1.1')
        self._open_license_and_check('OLDAP-1.2.json',
                                     self._find_licenses_matcher,
                                     'OLDAP-1.2')
        self._open_license_and_check('OLDAP-1.3.json',
                                     self._find_licenses_matcher,
                                     'OLDAP-1.3')
        self._open_license_and_check('OLDAP-1.4.json',
                                     self._find_licenses_matcher,
                                     'OLDAP-1.4')
        self._open_license_and_check('OLDAP-2.0.1.json',
                                     self._find_licenses_matcher,
                                     'OLDAP-2.0.1')
        self._open_license_and_check('OLDAP-2.0.json',
                                     self._find_licenses_matcher,
                                     'OLDAP-2')
        self._open_license_and_check('OLDAP-2.1.json',
                                     self._find_licenses_matcher,
                                     'OLDAP-2.1')
        self._open_license_and_check('OLDAP-2.2.1.json',
                                     self._find_licenses_matcher,
                                     'OLDAP-2.2.1')
        self._open_license_and_check('OLDAP-2.2.2.json',
                                     self._find_licenses_matcher,
                                     'OLDAP-2.2.2')
        self._open_license_and_check('OLDAP-2.2.json',
                                     self._find_licenses_matcher,
                                     'OLDAP-2.2')
        self._open_license_and_check('OLDAP-2.3.json',
                                     self._find_licenses_matcher,
                                     'OLDAP-2.3')
        self._open_license_and_check('OLDAP-2.4.json',
                                     self._find_licenses_matcher,
                                     'OLDAP-2.4')
        self._open_license_and_check('OLDAP-2.5.json',
                                     self._find_licenses_matcher,
                                     'OLDAP-2.5')
        self._open_license_and_check('OLDAP-2.6.json',
                                     self._find_licenses_matcher,
                                     'OLDAP-2.6')
        self._open_license_and_check('OLDAP-2.7.json',
                                     self._find_licenses_matcher,
                                     'OLDAP-2.7')
        self._open_license_and_check('OLDAP-2.8.json',
                                     self._find_licenses_matcher,
                                     'OLDAP-2.8')
        self._open_license_and_check('OML.json',
                                     self._find_licenses_matcher,
                                     'OML')
        self._open_license_and_check('OpenSSL.json',
                                     self._find_licenses_matcher,
                                     'OpenSSL or SSLeay')
        self._open_license_and_check('OPL-1.0.json',
                                     self._find_licenses_matcher,
                                     'OPL-1')
        self._open_license_and_check('OSET-PL-2.1.json',
                                     self._find_licenses_matcher,
                                     'OSET-PL-2.1')
        self._open_license_and_check('OSL-1.0.json',
                                     self._find_licenses_matcher,
                                     'OSL-1')
        self._open_license_and_check('OSL-1.1.json',
                                     self._find_licenses_matcher,
                                     'OSL-1.1')
        self._open_license_and_check('OSL-2.0.json',
                                     self._find_licenses_matcher,
                                     'OSL-2')
        self._open_license_and_check('OSL-2.1.json',
                                     self._find_licenses_matcher,
                                     'OSL-2.1')
        self._open_license_and_check('OSL-3.0.json',
                                     self._find_licenses_matcher,
                                     'OSL-3')

    def testSpdxP(self):
        self._open_license_and_check('PDDL-1.0.json',
                                     self._find_licenses_matcher,
                                     'PDDL-1')
        self._open_license_and_check('PHP-3.0.json',
                                     self._find_licenses_matcher,
                                     'PHP-3')
        self._open_license_and_check('PHP-3.01.json',
                                     self._find_licenses_matcher,
                                     'PHP-3.01')
        self._open_license_and_check('Plexus.json',
                                     self._find_licenses_matcher,
                                     'Plexus')
        self._open_license_and_check('PostgreSQL.json',
                                     self._find_licenses_matcher,
                                     'PostgreSQL')
        self._open_license_and_check('psfrag.json',
                                     self._find_licenses_matcher,
                                     'psfrag')
        self._open_license_and_check('psutils.json',
                                     self._find_licenses_matcher,
                                     'psutils')
        # The MIT/X11 part could do with a more specific name
        self._open_license_and_check('Python-2.0.json',
                                     self._find_licenses_matcher,
                                     'CNRI-Python or MIT/X11 or Python-2')

    def testSpdxQ(self):
        self._open_license_and_check('Qhull.json',
                                     self._find_licenses_matcher,
                                     'Qhull')
        self._open_license_and_check('QPL-1.0.json',
                                     self._find_licenses_matcher,
                                     'QPL-1')

    def testSpdxR(self):
        self._open_license_and_check('Rdisc.json',
                                     self._find_licenses_matcher,
                                     'Rdisc')
        self._open_license_and_check('RHeCos-1.1.json',
                                     self._find_licenses_matcher,
                                     'RHeCos-1.1')
        self._open_license_and_check('RPL-1.1.json',
                                     self._find_licenses_matcher,
                                     'RPL-1.1')
        self._open_license_and_check('RPL-1.5.json',
                                     self._find_licenses_matcher,
                                     'RPL-1.5')
        self._open_license_and_check('RPSL-1.0.json',
                                     self._find_licenses_matcher,
                                     'RPSL-1')
        self._open_license_and_check('RSA-MD.json',
                                     self._find_licenses_matcher,
                                     'RSA-MD')
        self._open_license_and_check('RSCPL.json',
                                     self._find_licenses_matcher,
                                     'RSCPL-1')
        self._open_license_and_check('Ruby.json',
                                     self._find_licenses_matcher,
                                     'Ruby')

    def testSpdxS(self):
        self._open_license_and_check('SAX-PD.json',
                                     self._find_licenses_matcher,
                                     'public-domain')
        self._open_license_and_check('Saxpath.json',
                                     self._find_licenses_matcher,
                                     'BSD-like-Saxpath')
        self._open_license_and_check('SCEA.json',
                                     self._find_licenses_matcher,
                                     'SCEA-1')
        self._open_license_and_check('Sendmail.json',
                                     self._find_licenses_matcher,
                                     'Sendmail')
        self._open_license_and_check('SGI-B-1.0.json',
                                     self._find_licenses_matcher,
                                     'SGI-B-1')
        self._open_license_and_check('SGI-B-1.1.json',
                                     self._find_licenses_matcher,
                                     'SGI-B-1.1')
        self._open_license_and_check('SGI-B-2.0.json',
                                     self._find_licenses_matcher,
                                     'SGI-B-2')
        self._open_license_and_check('SimPL-2.0.json',
                                     self._find_licenses_matcher,
                                     'SimPL-2')
        self._open_license_and_check('SISSL-1.2.json',
                                     self._find_licenses_matcher,
                                     'SISSL-1.2')
        self._open_license_and_check('SISSL.json',
                                     self._find_licenses_matcher,
                                     'SISSL-1.1')
        self._open_license_and_check('Sleepycat.json',
                                     self._find_licenses_matcher,
                                     'Sleepycat')
        self._open_license_and_check('SMLNJ.json',
                                     self._find_licenses_matcher,
                                     'MIT-like-SMLNJ')
        self._open_license_and_check('SMPPL.json',
                                     self._find_licenses_matcher,
                                     'SMPPL')
        self._open_license_and_check('SNIA.json',
                                     self._find_licenses_matcher,
                                     'SNIA-1')
        self._open_license_and_check('Spencer-86.json',
                                     self._find_licenses_matcher,
                                     'Spencer-86')
        self._open_license_and_check('Spencer-94.json',
                                     self._find_licenses_matcher,
                                     'Spencer-94')
        self._open_license_and_check('Spencer-99.json',
                                     self._find_licenses_matcher,
                                     'Spencer-99')
        self._open_license_and_check('SPL-1.0.json',
                                     self._find_licenses_matcher,
                                     'SPL-1')
        self._open_license_and_check('StandardML-NJ.json',
                                     self._find_licenses_matcher,
                                     'MIT-like-SMLNJ')
        self._open_license_and_check('SugarCRM-1.1.3.json',
                                     self._find_licenses_matcher,
                                     'SugarCRM-1.1.3')
        self._open_license_and_check('SWL.json',
                                     self._find_licenses_matcher,
                                     'SWL')

    def testSpdxT(self):
        self._open_license_and_check('TCL.json',
                                     self._find_licenses_matcher,
                                     'TCL')
        self._open_license_and_check('TMate.json',
                                     self._find_licenses_matcher,
                                     'BSD-like-TMate')
        self._open_license_and_check('TORQUE-1.1.json',
                                     self._find_licenses_matcher,
                                     'TORQUE-1.1')
        self._open_license_and_check('TOSL.json',
                                     self._find_licenses_matcher,
                                     'BSD-like-TOSL')

    def testSpdxU(self):
        self._open_license_and_check('Unicode-TOU.json',
                                     self._find_licenses_matcher,
                                     'Unicode-TOU')
        self._open_license_and_check('Unlicense.json',
                                     self._find_licenses_matcher,
                                     'Unlicense')
        self._open_license_and_check('UPL-1.0.json',
                                     self._find_licenses_matcher,
                                     'UPL-1')

    def testSpdxV(self):
        self._open_license_and_check('Vim.json',
                                     self._find_licenses_matcher,
                                     'Vim')
        self._open_license_and_check('VOSTROM.json',
                                     self._find_licenses_matcher,
                                     'VOSTROM')
        self._open_license_and_check('VSL-1.0.json',
                                     self._find_licenses_matcher,
                                     'VSL-1')

    def testSpdxW(self):
        # self._open_license_and_check('W3C-19980720.json',
        #                              self._find_licenses_matcher,
        #                              'W3C-19980720')
        self._open_license_and_check('W3C.json',
                                     self._find_licenses_matcher,
                                     'W3C',
                                     ignore={"standardLicenseHeader"})
        self._open_license_and_check('Watcom-1.0.json',
                                     self._find_licenses_matcher,
                                     'Watcom-1')
        self._open_license_and_check('Wsuipa.json',
                                     self._find_licenses_matcher,
                                     'Wsuipa')
        self._open_license_and_check('WTFPL.json',
                                     self._find_licenses_matcher,
                                     'WTFPL')
        self._open_license_and_check('WXwindows.json',
                                     self._find_licenses_matcher,
                                     'LGPL-2+ with WxWindows-3.1+ exception')

    def testSpdxX(self):
        self._open_license_and_check('X11.json',
                                     self._find_licenses_matcher,
                                     'X11')
        self._open_license_and_check('Xerox.json',
                                     self._find_licenses_matcher,
                                     'Xerox')
        self._open_license_and_check('XFree86-1.1.json',
                                     self._find_licenses_matcher,
                                     'XFree86-1.1')
        self._open_license_and_check('xinetd.json',
                                     self._find_licenses_matcher,
                                     'xinetd')
        self._open_license_and_check('Xnet.json',
                                     self._find_licenses_matcher,
                                     'Expat')
        self._open_license_and_check('xpp.json',
                                     self._find_licenses_matcher,
                                     'xpp')
        self._open_license_and_check('XSkat.json',
                                     self._find_licenses_matcher,
                                     'XSkat')

    def testSpdxY(self):
        self._open_license_and_check('YPL-1.0.json',
                                     self._find_licenses_matcher,
                                     'YPL-1')
        self._open_license_and_check('YPL-1.1.json',
                                     self._find_licenses_matcher,
                                     'YPL-1.1')

    def testSpdxZ(self):
        self._open_license_and_check('Zed.json',
                                     self._find_licenses_matcher,
                                     'Zed')
        self._open_license_and_check('Zend-2.0.json',
                                     self._find_licenses_matcher,
                                     'Zend-2')
        self._open_license_and_check('Zimbra-1.3.json',
                                     self._find_licenses_matcher,
                                     'Zimbra-1.3')
        self._open_license_and_check('Zimbra-1.4.json',
                                     self._find_licenses_matcher,
                                     'Zimbra-1.4')
        self._open_license_and_check('zlib-acknowledgement.json',
                                     self._find_licenses_matcher,
                                     'Zlib-acknowledgement')
        self._open_license_and_check('Zlib.json',
                                     self._find_licenses_matcher,
                                     'Zlib')
        self._open_license_and_check('ZPL-1.1.json',
                                     self._find_licenses_matcher,
                                     'ZPL-1.1')
        self._open_license_and_check('ZPL-2.0.json',
                                     self._find_licenses_matcher,
                                     'ZPL-2')
        self._open_license_and_check('ZPL-2.1.json',
                                     self._find_licenses_matcher,
                                     'ZPL-2.1')


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python3
# encoding: utf-8

# Unit tests for decopy
# Copyright (C) 2016 Margarita Manterola <marga@debian.org>
#
# License: ISC
#  Permission to use, copy, modify, and/or distribute this software for any
#  purpose with or without fee is hereby granted, provided that the above
#  copyright notice and this permission notice appear in all copies.
#  .
#  THE SOFTWARE IS PROVIDED "AS IS" AND ISC DISCLAIMS ALL WARRANTIES WITH
#  REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
#  AND FITNESS. IN NO EVENT SHALL ISC BE LIABLE FOR ANY SPECIAL, DIRECT,
#  INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
#  LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
#  OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
#  PERFORMANCE OF THIS SOFTWARE.

import unittest

from decopy import cmdoptions
from decopy import tree
from decopy import dep5
from decopy import datatypes


class TestCopyright(unittest.TestCase):

    def _processTree(self, path):
        '''Process the received tree and parse the copyright file.'''
        options = cmdoptions.process_options(['--no-progress', '--root', path])
        filetree = tree.RootInfo.build(options)
        copyright_ = dep5.Copyright.build(filetree, options)
        copyright_.process(filetree)
        return copyright_, options

    def testProcessEmpty(self):
        '''Parses a tree with no debian/copyright file.'''

        copyright_, _ = self._processTree('tests/testdata/two')

        # An empty copyright file should yield no groups after processing
        self.assertEqual(copyright_.groups, [])

    def testProcessMalformed(self):
        '''Parses a tree with a malformed debian/copyright file.'''

        copyright_, _ = self._processTree('tests/testdata/malformed')

        # An empty copyright file should yield no groups after processing
        self.assertEqual(copyright_.groups, [])

    def testProcessIncomplete(self):
        '''Parses a tree with an incomplete debian/copyright file.'''

        copyright_, options = self._processTree('tests/testdata/incomplete')

        # An incomplete copyright file should yield the incomplete groups after
        # processing
        expected_groups = [
            {
                'license': 'Unknown',
                'holders': [],
                'files':   ['debian/copyright'],
                'comment': '',
            },
            {
                'license': 'Unknown',
                'holders': [],
                'files':   ['test'],
                'comment': '',
            },
        ]
        self._checkSortedGroups(expected_groups, copyright_.groups, options)

    def testProcessSimple(self):
        '''Parses a debian/copyright with two entries (testdata/four).'''

        copyright_, options = self._processTree('tests/testdata/four')

        expected_groups = [
            {
                'license': 'LGPL-2+',
                'holders': ['2014-2016, Maximiliano Curia <maxy@debian.org>'],
                'files':   ['debian/compat', 'debian/copyright'],
                'comment': '',
            },
            {
                'license': 'GPL-2+',
                'holders': ['2016, Jane Doe <janedoe@example.com>'],
                'files':   ['test'],
                'comment': '',
            },
        ]

        self._checkSortedGroups(expected_groups, copyright_.groups, options)

    def testProcessComplex(self):
        '''Parses a debian/copyright with many entries (testdata/three).'''

        copyright_, options = self._processTree('tests/testdata/three')

        expected_groups = [
            {
                'license': 'LGPL-2+',
                'holders': ['2014-2016, Maximiliano Curia <maxy@debian.org>'],
                'files':   ['debian/compat', 'debian/copyright',
                            'debian/rules'],
                'comment': 'No real packaging, just empty files.',
            },
            {
                'license': 'GPL-2+',
                'holders': ['1989-2003, Free Software Foundation, Inc'],
                'files':   ['acinclude.m4'],
                'comment': '',
            },
            {
                'license': 'MIT/X11',
                'holders': ['1993-2004, Clara Oswald <clara@example.com>'],
                'files':   ['src/do-something.m', 'src/do-something.py'],
                'comment': '',
            },
            {
                'license': 'GPL-2+',
                'holders': ['2016, Jane Doe <janedoe@example.com>'],
                'files':   ['COPYING', 'data/file.txt', 'src/main.py', 'src/module.py'],
                'comment': '',
            },
        ]

        self._checkSortedGroups(expected_groups, copyright_.groups, options)

    def _checkSortedGroups(self, expected_groups, groups, options):
        for i, group in enumerate(groups):
            key = group.get_key(options)
            expected = expected_groups[i]
            with self.subTest(msg='Checking "{}"'.format(key)):
                self.assertEqual(
                    list(group.copyrights.sorted_members()),
                    expected['holders'])
                self.assertEqual(group.license, expected['license'])
                self.assertEqual(
                    group.files.sorted_members(), expected['files'])
                self.assertEqual(group.get_comments(), expected['comment'])

    def _processAndRemove(self, path, opts):
        '''Lists the tree, parses the copyright, reads all files, removes
        misplaced.'''
        options = cmdoptions.process_options(['--no-progress', '--root', path] + opts)
        filetree = tree.RootInfo.build(options)
        copyright_ = dep5.Copyright.build(filetree, options)
        copyright_.process(filetree)
        filetree.process(options)
        # This function removes files from the groups when they should no longer
        # be there.  It does not add them to new or existing groups.  So the
        # effect that we will see is that some files or groups go "missing".
        copyright_.remove_misplaced_files(options)
        return copyright_, options

    def _checkExpectedGroups(self, expected_groups, groups):
        '''Checks that the received and expected groups match.'''
        # Check that we got all expected groups
        self.longMessage = False
        for key in expected_groups:
            with self.subTest(msg='Checking "{}"'.format(key)):
                self.assertIn(key, groups,
                                msg='missing expected group: {}'.format(key))
        for key in groups:
            with self.subTest(msg='Checking "{}"'.format(key)):
                self.assertIn(key, expected_groups,
                                msg='unexpected group: {}'.format(key))
        self.longMessage = True

        # Check that the groups are what we expected
        for key, group in groups.items():
            with self.subTest(msg='Checking "{}"'.format(key)):
                expected = expected_groups[key]
                self.assertEqual(
                    list(group.copyrights.sorted_members()),
                    expected['holders'])
                self.assertEqual(group.license, expected['license'])
                self.assertEqual(
                    group.files.sorted_members(), expected['files'])
                self.assertEqual(group.get_comments(), expected['comment'])

    def testRemoveMisplaced(self):
        '''Removes files from copyright if they were wrong (testdata/three).'''

        copyright_, options = self._processAndRemove('tests/testdata/three', [])

        expected_groups = {
            ('LGPL-2+', None, 'debian'): {
                'license': 'LGPL-2+',
                'holders': ['2014-2016, Maximiliano Curia <maxy@debian.org>'],
                'files':   ['debian/compat', 'debian/copyright',
                            'debian/rules'],
                'comment': 'No real packaging, just empty files.',
            },
            ('GPL-2+', None, ''): {
                'license': 'GPL-2+',
                'holders': ['1989-2003, Free Software Foundation, Inc',
                            '2016, Jane Doe <janedoe@example.com>'],
                'files':   ['COPYING', 'acinclude.m4', 'src/main.py', 'src/module.py'],
                'comment': '',
            },
            ('MIT/X11', None, ''): {
                'license': 'MIT/X11',
                'holders': ['1993-2004, Clara Oswald <clara@example.com>'],
                'files':   ['src/do-something.m', 'src/do-something.py'],
                'comment': '',
            },
        }
        groups = copyright_.get_group_dict(options)
        self._checkExpectedGroups(expected_groups, groups)

    def testRemoveMisplacedCopyright(self):
        '''Removes files if they were wrong, grouping by copyright.'''

        copyright_, options = self._processAndRemove(
            'tests/testdata/three', ['--group-by', 'copyright'])

        expected_groups = {
            ('LGPL-2+', ('maxy@debian.org',), 'debian'): {
                'license': 'LGPL-2+',
                'holders': ['2014-2016, Maximiliano Curia <maxy@debian.org>'],
                'files':   ['debian/compat', 'debian/copyright',
                            'debian/rules'],
                'comment': 'No real packaging, just empty files.',
            },
            ('GPL-2+', ('janedoe@example.com',), ''): {
                'license': 'GPL-2+',
                'holders': ['2016, Jane Doe <janedoe@example.com>'],
                'files':   ['COPYING', 'src/main.py', 'src/module.py'],
                'comment': '',
            },
            ('MIT/X11', ('clara@example.com',), ''): {
                'license': 'MIT/X11',
                'holders': ['1993-2004, Clara Oswald <clara@example.com>'],
                'files':   ['src/do-something.m', 'src/do-something.py'],
                'comment': '',
            },
            ('GPL-2+', ('Free Software Foundation, Inc',), ''): {
                'license': 'GPL-2+',
                'holders': ['1989-2003, Free Software Foundation, Inc'],
                'files':   ['acinclude.m4'],
                'comment': '',
            },
        }
        groups = copyright_.get_group_dict(options)
        self._checkExpectedGroups(expected_groups, groups)

    def testRemoveMisplacedPartial(self):
        '''Removes files if they were wrong, grouping by copyright.'''

        copyright_, options = self._processAndRemove(
            'tests/testdata/three',
            ['--mode', 'partial', 'tests/testdata/three/debian'])

        expected_groups = {
            ('LGPL-2+', None, 'debian'): {
                'license': 'LGPL-2+',
                'holders': ['2014-2016, Maximiliano Curia <maxy@debian.org>'],
                'files':   ['debian/compat', 'debian/copyright',
                            'debian/rules'],
                'comment': 'No real packaging, just empty files.',
            },
            ('GPL-2+', None, ''): {
                'license': 'GPL-2+',
                'holders': ['1989-2003, Free Software Foundation, Inc'],
                'files':   [],
                'comment': '',
            },
            ('MIT/X11', None, ''): {
                'license': 'MIT/X11',
                'holders': ['1993-2004, Clara Oswald <clara@example.com>'],
                'files':   [],
                'comment': '',
            },
        }

        groups = copyright_.get_group_dict(options)
        self._checkExpectedGroups(expected_groups, groups)


class TestGroup(unittest.TestCase):

    def testUnknownLicense(self):
        g = dep5.Group()
        self.assertEqual(g.license, datatypes.UNKNOWN)
        cg = tree.CopyrightGroup()
        cg.add(datatypes.CopyrightHolder('', 'foo@example.com', ''))
        g.add_copyrights(cg)
        self.assertEqual(g.license, datatypes.UNKNOWN_COPYRIGHTED)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python3
# encoding: utf-8

# Unit tests for decopy
# Copyright (C) 2018 Maximiliano Curia <maxy@debian.org>
#
# License: ISC
#  Permission to use, copy, modify, and/or distribute this software for any
#  purpose with or without fee is hereby granted, provided that the above
#  copyright notice and this permission notice appear in all copies.
#  .
#  THE SOFTWARE IS PROVIDED "AS IS" AND ISC DISCLAIMS ALL WARRANTIES WITH
#  REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
#  AND FITNESS. IN NO EVENT SHALL ISC BE LIABLE FOR ANY SPECIAL, DIRECT,
#  INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
#  LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
#  OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
#  PERFORMANCE OF THIS SOFTWARE.

import glob
import json
import time
import sys

try:
    from tqdm import tqdm
except ImportError:
    def tqdm(*a, **kw):
        if a:
            return a[0]
        return kw.get('iterable', None)

from decopy import matchers


class SpeedCheck:

    def run(self):
        self.benchmark_licenses_res()
        self.benchmark_clean_comments()
        self.benchmark_copyright()
        self.benchmark_resubs()
        return 0

    def load_all_licenses(self):
        texts = []
        for filename in glob.glob('tests/testdata/licenses/*.json'):
            with open(filename) as file_:
                license_ = json.load(file_)
            for key in ('licenseText', 'standardLicenseHeader',
                        'licenseExceptionText'):
                if key not in license_:
                    continue
                texts.append(license_[key])
        return texts

    def benchmark_licenses_res(self):

        texts = self.load_all_licenses()
        times = []

        for i, pattern in tqdm(enumerate(matchers.COMPILED_RES),
                               desc='Timing license res',
                               total=len(matchers.COMPILED_RES),
                               dynamic_ncols=True):
            start = time.time()
            for text in texts:
                pattern.search(text)
            elapsed = time.time() - start
            times.append((elapsed, i))

        base, amount = 0, 0
        interesting = times[base:base + amount]
        if interesting:
            print("Sum of current selection: ",
                  sum(elapsed for elapsed, _ in interesting))
        times.sort()

        for elapsed, i in [times[0]] + interesting + times[-5:]:
            license_re = matchers.LICENSES_RES[i]
            print("Time for {} ({}): {}\n"
                  "{}".format(i, license_re.license, elapsed, license_re.re))

    def benchmark_resubs(self):

        texts = self.load_all_licenses()
        times = []
        resubs = (matchers.ARTISTIC_SUBS
                  + matchers.BSD_SUBS
                  + matchers.CC_BY_SUBS
                  + matchers.CECILL_SUBS
                  + matchers.CNRI_SUBS
                  + matchers.EFL_SUBS
                  + matchers.GNU_VERSION_SUBS
                  + matchers.GNU_EXCEPTION_SUBS
                  + matchers.GFDL_SUBS
                  + matchers.LPPL_SUBS
                  + matchers.MIT_SUBS
                  + matchers.ZPL_SUBS
                  )

        for i, resub in tqdm(enumerate(resubs),
                             desc='Timing resubs',
                             total=len(resubs),
                             dynamic_ncols=True):
            start = time.time()
            for text in texts:
                resub.pattern.subn(resub.repl, text, 1)
            elapsed = time.time() - start
            times.append((elapsed, i))

        base, amount = 0, 0
        interesting = times[base:base + amount]
        if interesting:
            print("Sum of current selection: ",
                  sum(elapsed for elapsed, _ in interesting))
        times.sort()

        for elapsed, i in [times[0]] + interesting + times[-5:]:
            resub = resubs[i]
            print("Time for {} (repl: {}): {}".format(i, resub.repl, elapsed))

    def benchmark_clean_comments(self):

        texts = self.load_all_licenses()
        times = []

        for i, sub in tqdm(enumerate(matchers.COMMENTS_SUBS),
                           desc='Timing comment res',
                           total=len(matchers.COMMENTS_SUBS),
                           dynamic_ncols=True):
            start = time.time()
            for text in texts:
                sub.pattern.sub(sub.repl, text)
            elapsed = time.time() - start
            times.append((elapsed, i))

        base, amount = 0, 0
        interesting = times[base:base + amount]
        if interesting:
            print("Sum of current selection: ",
                  sum(elapsed for elapsed, _ in interesting))
        times.sort()

        for elapsed, i in [times[0]] + interesting + times[-5:]:
            print("Time for {} : {}".format(i, elapsed))

    def benchmark_copyright(self):

        texts = self.load_all_licenses()
        times = []

        patterns = (matchers.COPYRIGHT_PRE_IGNORE_RES
                    + matchers.COPYRIGHT_POST_IGNORE_RES
                    + (matchers.COPYRIGHT_PRE_INDICATOR_RE,
                       matchers.COPYRIGHT_INDICATOR_RE))

        for i, pattern in tqdm(enumerate(patterns),
                               desc='Timing copyright res',
                               total=len(patterns),
                               dynamic_ncols=True):
            start = time.time()
            for text in texts:
                pattern.findall(text)
            elapsed = time.time() - start
            times.append((elapsed, i))

        base, amount = 0, 0
        # len(matchers.COPYRIGHT_PRE_IGNORE_RES) + 1, \
        #     len(matchers.COPYRIGHT_POST_IGNORE_RES)
        interesting = times[base:base + amount]
        if interesting:
            print("Sum of current selection: ",
                  sum(elapsed for elapsed, _ in interesting))
        times.sort()

        for elapsed, i in [times[0]] + interesting + times[-5:]:
            print("Time for {} : {}".format(i, elapsed))


if __name__ == '__main__':
    check = SpeedCheck()
    sys.exit(check.run())
